@@ -1,5 +1,46 @@
 package Data::Xslate;
-$Data::Xslate::VERSION = '0.01';
+$Data::Xslate::VERSION = '0.02';
+=head1 NAME
+
+Data::Xslate - Templatize your data.
+
+=head1 SYNOPSIS
+
+    use Data::Xslate;
+    
+    my $xslate = Data::Xslate->new();
+    
+    my $data = $xslate->render(
+        {
+            user => {
+                login => 'john',
+                email => '<: $login :>@example.com',
+                name  => 'John',
+            },
+            email => {
+                to      => '=user.email',
+                subject => 'Hello <: $user.name :>!',
+            },
+            'email.from=' => 'george@example.com',
+        },
+    );
+    
+    # This would produce the following:
+    # $data->{email} = {
+    #     to => 'john@example.com',
+    #     from => 'george@example.com',
+    #     subject => 'Hello John!',
+    # }
+
+=head1 DESCRIPTION
+
+This module provides a syntax for templatizing data structures.
+
+The most likely use-case is adding some flexibility to configuration
+files.
+
+=cut
+
 use Text::Xslate;
 use Types::Standard -types;
 use Types::Common::String -types;
@@ -12,9 +53,10 @@ use namespace::clean;
 # A tied-hash class used to expose the data as the Xslate
 # vars when processing the data.
 {
-    package Data::Xslate::Vars;
-$Data::Xslate::Vars::VERSION = '0.01';
-use base 'Tie::Hash';
+    package # NO INDEX CPAN!
+        Data::Xslate::Vars;
+
+    use base 'Tie::Hash';
 
     sub TIEHASH {
         my ($class, $sub) = @_;
@@ -28,15 +70,6 @@ use base 'Tie::Hash';
     }
 }
 
-# State variables, only used during local() calls to maintane
-# state in recursive function calls.
-our $XSLATE;
-our $VARS;
-our $ROOT;
-our $NODES;
-our $SUBSTITUTION_TAG;
-our $PATH_FOR_XSLATE;
-
 around BUILDARGS => sub{
     my $orig = shift;
     my $class = shift;
@@ -46,6 +79,8 @@ around BUILDARGS => sub{
 
     my @expected_args = qw(
         substitution_tag
+        nested_key_tag
+        key_separator
     );
     foreach my $arg (@expected_args) {
         next if !exists $xslate_args->{$arg};
@@ -60,12 +95,6 @@ has _xslate_args => (
     is => 'ro',
 );
 
-has substitution_tag => (
-    is      => 'ro',
-    isa     => NonEmptySimpleStr,
-    default => '=:',
-);
-
 has _xslate => (
     is       => 'lazy',
     init_arg => undef,
@@ -84,6 +113,96 @@ sub _build__xslate {
     );
 }
 
+=head1 ARGUMENTS
+
+Any arguments you pass to C<new>, which this class does not directly
+handle, will be used when creating the underlying L<Text::Xslate> object.
+So, any arguments which L<Text::Xslate> supports may be set.  For example:
+
+    my $xslate = Data::Xslate->new(
+        substitution_tag => ']]', # A Data::Xslate argument.
+        verbose          => 2,    # A Text::Xslate option.
+    );
+
+=head2 substitution_tag
+
+The string to look for at the beginning of any string value which
+signifies L</SUBSTITUTION>.  Defaults to C<=>.  This is used in
+data like this:
+
+    { a=>{ b=>2 }, c => '=a.b' }
+
+=cut
+
+has substitution_tag => (
+    is      => 'ro',
+    isa     => NonEmptySimpleStr,
+    default => '=',
+);
+
+=head2 nested_key_tag
+
+The string to look for at the end of any key which signifies
+L</NESTED KEYS>.  Defaults to C<=>.  This is used in data
+like this:
+
+    { a=>{ b=>2 }, 'a.c=' => 3 }
+
+=cut
+
+has nested_key_tag => (
+    is      => 'ro',
+    isa     => NonEmptySimpleStr,
+    default => '=',
+);
+
+=head2 key_separator
+
+The string which will be used between keys.  The default is a dot (C<.>)
+which looks like this:
+
+    { a=>{ b=>2 }, c => '=a.b' }
+
+Whereas, for example, if you changed the C<key_separator> to a forward
+slash it would look like this:
+
+    { a=>{ b=>2 }, c => '=a/b' }
+
+Which actually looks pretty good when you do an absolute, rather than
+relative, key:
+
+    { a=>{ b=>2 }, c => '=/a/b' }
+
+=cut
+
+has key_separator => (
+    is      => 'ro',
+    isa     => NonEmptySimpleStr,
+    default => '.',
+);
+
+=head1 METHODS
+
+=head2 render
+
+    my $data_out = $xslate->render( $data_in );
+
+Processes the data and returns new data.  The passed in data is not
+modified.
+
+=cut
+
+# State variables, only used during local() calls to maintane
+# state in recursive function calls.
+our $XSLATE;
+our $VARS;
+our $ROOT;
+our $NODES;
+our $SUBSTITUTION_TAG;
+our $NESTED_KEY_TAG;
+our $KEY_SEPARATOR;
+our $PATH_FOR_XSLATE;
+
 sub render {
     my ($self, $data) = @_;
 
@@ -98,6 +217,8 @@ sub render {
     local $ROOT = $data;
     local $NODES = {};
     local $SUBSTITUTION_TAG = $self->substitution_tag();
+    local $NESTED_KEY_TAG = $self->nested_key_tag();
+    local $KEY_SEPARATOR = $self->key_separator();
 
     return _evaluate_node( 'root' => $data );
 }
@@ -109,7 +230,7 @@ sub _evaluate_node {
 
     if (!ref $node) {
         if (defined $node) {
-            if ($node =~ m{^$SUBSTITUTION_TAG\s*(.+?)\s*$}) {
+            if ($node =~ m{^\Q$SUBSTITUTION_TAG\E\s*(.+?)\s*$}) {
                 $node = _find_node( $1, $path );
             }
             else {
@@ -122,12 +243,13 @@ sub _evaluate_node {
     elsif (ref($node) eq 'HASH') {
         $NODES->{$path} = $node;
         foreach my $key (sort keys %$node) {
-            my $sub_path = "$path.$key";
-            if ($key =~ m{\.}) {
+            if ($key =~ m{^(.*)\Q$NESTED_KEY_TAG\E$}) {
+                my $sub_path = "$path$KEY_SEPARATOR$1";
                 my $value = delete $node->{$key};
                 _set_node( $sub_path, $value );
             }
             else {
+                my $sub_path = "$path$KEY_SEPARATOR$key";
                 $node->{$key} = _evaluate_node( $sub_path, $node->{$key} );
             }
         }
@@ -135,7 +257,7 @@ sub _evaluate_node {
     elsif (ref($node) eq 'ARRAY') {
         $NODES->{$path} = $node;
         @$node = (
-            map { _evaluate_node( "$path.$_" => $node->[$_] ) }
+            map { _evaluate_node( "$path$KEY_SEPARATOR$_" => $node->[$_] ) }
             (0..$#$node)
         );
     }
@@ -149,13 +271,13 @@ sub _evaluate_node {
 sub _load_node {
     my ($path) = @_;
 
-    my @parts = split(/\./, $path);
+    my @parts = split(/\Q$KEY_SEPARATOR\E/, $path);
     my $built_path = shift( @parts ); # root
 
     my $node = $ROOT;
     while (@parts) {
         my $key = shift( @parts );
-        $built_path .= ".$key";
+        $built_path .= "$KEY_SEPARATOR$key";
 
         if (ref($node) eq 'HASH') {
             return undef if !exists $node->{$key};
@@ -176,18 +298,18 @@ sub _load_node {
 sub _find_node {
     my ($path, $from_path) = @_;
 
-    if ($path =~ m{^\.(.+)}) {
+    if ($path =~ m{^\Q$KEY_SEPARATOR\E(.+)}) {
         $path = $1;
-        $from_path = 'root.foo';
+        $from_path = "root${KEY_SEPARATOR}root_sub_key_that_is_not_used_for_absolute_keys";
     }
 
-    my @parts = split(/\./, $from_path);
+    my @parts = split(/\Q$KEY_SEPARATOR\E/, $from_path);
     pop( @parts );
 
     while (@parts) {
-        my $sub_path = join('.', @parts);
+        my $sub_path = join($KEY_SEPARATOR, @parts);
 
-        my $node = _load_node( "$sub_path.$path" );
+        my $node = _load_node( "$sub_path$KEY_SEPARATOR$path" );
         return $node if $node;
 
         pop( @parts );
@@ -204,14 +326,14 @@ sub _find_node_for_xslate {
 sub _set_node {
     my ($path, $value) = @_;
 
-    my @parts = split(/\./, $path);
+    my @parts = split(/\Q$KEY_SEPARATOR\E/, $path);
     my $built_path = shift( @parts ); # root
     my $last_part = pop( @parts );
 
     my $node = $ROOT;
     while (@parts) {
         my $key = shift( @parts );
-        $built_path .= ".$key";
+        $built_path .= "$KEY_SEPARATOR$key";
 
         if (ref($node) eq 'HASH') {
             return 0 if !exists $node->{$key};
@@ -242,127 +364,146 @@ sub _set_node {
 1;
 __END__
 
-=head1 NAME
-
-Data::Xslate - Templatize your data.
-
-=head1 SYNOPSIS
-
-    use Data::Xslate;
-    
-    my $xslate = Data::Xslate->new();
-    
-    my $new_data = $xslate->render(
-        {
-            user => {
-                login => 'john',
-                email => '<: $login :>@example.com',
-                name  => 'John',
-            },
-            email => {
-                to      => '=:user.email',
-                subject => 'Hello <: $user.name :>!',
-            },
-        },
-    );
-
-=head1 DESCRIPTION
-
-This module provides a syntax for templatizing data structures.
-
-The most likely use-case is adding some flexibility to configuration
-files.
-
-=head1 SUBSTITUTION
-
-    {
-        foo => 14,
-        bar => '=:foo',
-    }
-    # { foo=>14, bar=>14 }
-
-This injects the target value.  This can be used for any data type.  For
-example we can substitute an array:
-
-    {
-        foo => [1,2,3],
-        bar => '=:foo',
-    }
-    # { foo=>[1,2,3], bar=>[1,2,3] }
-
 =head1 TEMPLATING
 
+The most powerful feature by far is templating, where you can
+use L<Text::Xslate> in you values.
+
     {
         foo => 'green',
         bar => 'It is <: $foo :>!',
     }
     # { foo=>'green', bar=>'It is green!' }
 
-The syntax for templating is provided by L<Text::Xslate>, so
-there is a lot of power here including being able to do math
-and string mangling.
+There is a lot you can do with this beyond simply including values
+from other keys:
 
-=head1 SCOPE
+    {
+        prod => 1,
+        memcached_host => '<: if $prod { :>memcached.example.com<: } else { :>127.0.0.1<: } :>',
+    }
 
-When using either L</SUBSTITUTION> or L</TEMPLATING> you specify a key to be
-included.  This key is found using scope-aware rules where the key is searched for
-in a similar fashion to how you'd expect when dealing with lexical variables in
-programming.
+Values in arrays are also processed for templating:
 
-For example, you can refer to a key in the same scope:
+    {
+        ceo_name => 'Sara',
+        employees => [
+            '<: $ceo_name :>',
+            'Fred',
+            'Alice',
+        ],
+    }
 
-    { a=>1, b=>'=:a' }
+Data structures of any arbitrary depth and complexity are handled
+correctly, and keys from any level can be referred to following
+the L</SCOPE> rules.
 
-You may refer to a key in a lower scope:
+=head1 SUBSTITUTION
 
-    { a=>{ b=>1 }, c=>'=:a.b' }
+Substituion allows you to retrieve a value from one key and use it
+as the value for the current key.  To do this your hash or array
+value must start with the L</substitution_tag> (defaults to C<=>):
 
-You may refer to a key in a higher scope:
+    {
+        foo => 14,
+        bar => '=foo',
+    }
+    # { foo=>14, bar=>14 }
 
-    { a=>{ b=>'=:c' }, c=>1 }
+While the above could just have been written using templating:
 
-You may refer to a key in a higher scope that is nested:
+    {
+        foo => 14,
+        bar => '<: $foo :>',
+    }
 
-    { a=>{ b=>'=:c.d' }, c=>{ d=>1 } }
+But, templating only works with strings.  Substitution becomes vital
+when you want to substitute an array or hash:
 
-The logic behind this is pretty flexible, so more complex use cases will
-just work like you would expect.
+    {
+        foo => [1,2,3],
+        bar => '=foo',
+    }
+    # { foo=>[1,2,3], bar=>[1,2,3] }
 
-If you'd rather avoid this scoping you can prepend any key with a dot, C<.>, and
-it will be looked for at the root hash of the config tree only.
+The keys in substitution follow the L</SCOPE> rules.
 
 =head1 NESTED KEYS
 
-When setting a key value the key can point deeper into the structure by separating keys with
-a dot, C<.>.  Consider this:
+When setting a key value the key can point deeper into the structure by
+separating keys with the L</key_separator> (defaults to a dot, C<.>),
+and ending the key with the L</nested_key_tag> (defaults to C<=>).
+Consider this:
 
-    { a=>{ b=>1 }, 'a.b'=>2 }
+    { a=>{ b=>1 }, 'a.b=' => 2 }
 
 Which produces:
 
     { a=>{ b=>2 } }
 
-=head1 ARGUMENTS
+So, nested keys are a way to set values in other data structures.  This
+feature is very handy when you are merging data structures from different
+sources and one data structure will override a subset of values in the
+other.
 
-Any arguments you pass to C<new>, which this class does not directly
-handle, will be used when creating the L</xslate> object.  So, any
-arguments which L<Text::Xslate> supports may be set.  For example:
+=head1 KEY PATHS
 
-    my $xslate = Data::Xslate->new(
-        substitution_tag => ']]', # A Data::Xslate argument.
-        verbose          => 2,    # A Text::Xslate option.
-    );
+When referring to other values in L</TEMPLATING>, L</SUBSTITUTION>, or
+L</NESTED KEYS> you are specifying a path made up of keys for this module
+to walk and find a value to retrieve.
 
-=head2 substitution_tag
+So, when you specify a key path such as C<foo.bar> you are looking for a hash
+with the key C<foo> who's value is a hash and then retrieving the value
+of the C<bar> key in it.
 
-The string to look for at the beginning of any string value which
-signifies L</SUBSTITUTION>.  Defaults to C<=:>.
+Arrays are fully supported in these key paths so that if you specify
+a key path such as C<bar.0> you are looking for a hash with the C<bar>
+key whose value is an array, and then the first value in the array is
+fetched.
 
-=head1 METHODS
+Note that the above example assume that L</key_separator> is a dot (C<.>),
+the default.
 
-=head2 render
+=head1 SCOPE
 
-    my $data_out = $xslate->render( $data_in );
+When using either L</SUBSTITUTION> or L</TEMPLATING> you specify a key to be
+acted on.  This key is found using scope-aware rules where the key is searched for
+in a similar fashion to how you'd expect when dealing with lexical variables in
+programming.
+
+For example, you can refer to a key in the same scope:
+
+    { a=>1, b=>'=a' }
+
+You may refer to a key in a lower scope:
+
+    { a=>{ b=>1 }, c=>'=a.b' }
+
+You may refer to a key in a higher scope:
+
+    { a=>{ b=>'=c' }, c=>1 }
+
+You may refer to a key in a higher scope that is nested:
+
+    { a=>{ b=>'=c.d' }, c=>{ d=>1 } }
+
+The logic behind this is pretty flexible, so more complex use cases will
+just work like you would expect.
+
+If you'd rather avoid this scoping you can prepend any key with the L</key_separator>
+(defaults to a dot, C<.>), and it will be looked for at the root of the config data
+only.
+
+In the case of templating a special C<node> function is provided which
+will allow you to retrieve an absolute key.  For example these two lines
+would do the same thing (printing out a relative key value):
+
+    <: $foo.bar :>
+    <: node("foo.bar") :>
+
+But if you wanted to refer to an absolute key you'd have to do this:
+
+    <: node(".foo.bar") :>
 
 =head1 AUTHOR
 
